@@ -1,5 +1,5 @@
 # Archive.pm - This module does the archiving job for a mailing lists.
-# RCS Identication ; $Revision: 1.7 $ ; $Date: 2001/11/30 10:39:09 $ 
+# RCS Identication ; $Revision: 1.9 $ ; $Date: 2004/03/17 10:53:04 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,6 +26,7 @@ use POSIX;
 use Log;
 use strict;
 
+my $serial_number = 0; # incremented on each archived mail
 
 ## RCS identification.
 
@@ -56,7 +57,8 @@ sub outgoing {
     my @now  = localtime(time);
 #    my $prefix= sprintf("%04d-%02d-%02d-%02d-%02d-%02d",1900+$now[5],$now[4]+1,$now[3],$now[2],$now[1],$now[0]);
 #    my $filename = "$dir"."/"."$prefix-$listname";
-    my $filename = sprintf '%s/%s.%d.%d', $dir, $listname, time, $$;
+    my $filename = sprintf '%s/%s.%d.%d.%d', $dir, $listname, time, $$, $serial_number;
+    $serial_number = ($serial_number+1)%100000;
     unless ( open(OUT, "> $filename")) {
 	do_log('info',"error unable open outgoing dir $dir for list $listname");
 	return undef;
