@@ -1,5 +1,5 @@
 # Command.pm - this module does the mail commands processing
-# RCS Identication ; $Revision: 1.79.2.3 $ ; $Date: 2003/07/17 07:34:57 $ 
+# RCS Identication ; $Revision: 1.87.2.1 $ ; $Date: 2004/05/14 12:11:18 $ 
 
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -71,10 +71,6 @@ my $sender = '';
 my $time_command;
 my $msg_file;
 
-## Caution : if this regexp changes (more/less parenthesis), then regexp using it should 
-## also be changed
-my $email_regexp = '([\w\-\_\.\/\+\=]+|\".*\")\@[\w\-]+(\.[\w\-]+)+';
-
 ## Parse the command and call the adequate subroutine with
 ## the arguments to the command.
 sub parse {
@@ -953,7 +949,7 @@ sub add {
 
     do_log('debug', 'Commands::add(%s,%s)', $what,$sign_mod );
 
-    $what =~ /^(\S+)\s+($email_regexp)(\s+(.+))?\s*$/;
+    $what =~ /^(\S+)\s+($tools::regexp{'email'})(\s+(.+))?\s*$/;
     my($which, $email, $comment) = ($1, $2, $6);
     my $auth_method ;
 
@@ -1375,7 +1371,7 @@ sub del {
 
     do_log('debug', 'Commands::del(%s,%s)', $what,$sign_mod);
 
-    $what =~ /^(\S+)\s+($email_regexp)\s*/;
+    $what =~ /^(\S+)\s+($tools::regexp{'email'})\s*/;
     my($which, $who) = ($1, $2);
     my $auth_method;
     
@@ -1642,25 +1638,20 @@ sub distribute {
     ## encrypted message
     if ($message->{'smime_crypted'}) {
 	$is_crypted = 'smime_crypted';
-	$file = '_ALTERED_';
-	($msg, $file) = ($message->{'decrypted_msg'}, $message->{'decrypted_msg_as_string'});
-	unless (defined($msg)) {
-	    do_log('debug','unable to decrypt message');
-	    ## xxxxx traitement d'erreur ?
-	    return undef;
-	};
-	$hdr = $msg->head;
-	do_log('debug2', "message successfully decrypted");
+    }else {
+	$is_crypted = 'not_crypted';
     }
 
     $hdr->add('X-Validation-by', $sender);
 
     ## Distribute the message
-    my $numsmtp;
-    unless ($numsmtp = $list->distribute_msg($msg, $bytes, $file, $is_crypted)) {
+    my $numsmtp =$list->distribute_msg($message);
+    unless (defined $numsmtp) {
 	return undef;
     }
-
+    unless ($numsmtp) {
+	do_log('info', 'Message for %s from %s accepted but all subscribers use digest,nomail or summary',$which, $sender);
+    } 
     do_log('info', 'Message for %s from %s accepted (%d seconds, %d sessions), size=%d',
 	   $which, $sender, time - $start_time, $numsmtp, $bytes);
 
@@ -1722,7 +1713,7 @@ sub confirm {
     my $action = &List::request_action('send','md5',$robot,
 				       {'listname' => $name, 
 					'sender' => $sender ,
-					'msg' => $msg});
+					'message' => $message});
 
     return undef
 	unless (defined $action);
@@ -1760,8 +1751,9 @@ sub confirm {
 	$hdr->add('X-Validation-by', $sender);
 	
 	## Distribute the message
-	my $numsmtp;
-	unless ($numsmtp = $list->distribute_msg($msg, $bytes, $file, $is_crypted)) {
+	my $numsmtp = $list->distribute_msg($message);
+	unless (defined $numsmtp) {
+	    do_log('info','Unable to send message to list %s', $list->{'name'});
 	    return undef;
 	}
 
@@ -1998,11 +1990,12 @@ sub expire {
 		return undef;
 }
 	    do {
-		next unless ($user->{'date'} < $limitday);
-		push @msg::report, "   $user->{'email'}\n";      
-		&mail::mailback(\@msgexp, 
-				{'Subject' => sprintf(Msg(6, 21, "Renewal of your subscription to %s"), $name)},
-				'sympa', $user->{'email'}, $user->{'email'}, $robot);
+		if ($user->{'update_date'} < $limitday){
+		    push @msg::report, " $user->{'email'}\n";
+		    &mail::mailback(\@msgexp, 
+				    {'Subject' => sprintf(Msg(6, 21, "Renewal of your subscription to %s"), $name)},
+				    'sympa', $user->{'email'}, $user->{'email'}, $robot);
+		}
 	    } while ($user = $list->get_next_user());
 
 	    push @msg::report, "\n";
@@ -2141,10 +2134,11 @@ sub expireindex {
 }
 
         do {
-	    next unless ($user->{'date'} < $limitday);
-	    push @msg::report, "," if ($temp==1);
-	    push @msg::report, " $user->{'email'} ";
-	    $temp = 1 if ($temp==0);
+	    if ($user->{'update_date'} < $limitday){
+		push @msg::report, "," if ($temp==1);
+		push @msg::report, " $user->{'email'} ";
+		$temp = 1 if ($temp==0);
+	    }
 	} while ($user = $list->get_next_user());
 	push @msg::report, "\n\n";
 	push @msg::report, sprintf Msg(6, 58, "If you want to delete these subscribers, please use the following commands :\n");
@@ -2155,8 +2149,9 @@ sub expireindex {
 	}
 
 	do {
-	    next unless ($user->{'date'} < $limitday);
-	    push @msg::report, sprintf "DEL   $name   $user->{'email'}\n";
+	    if ($user->{'update_date'} < $limitday){
+		push @msg::report, sprintf "DEL $name $user->{'email'}\n";
+	    }
 	} while ($user = $list->get_next_user());
 
 	do_log('info', 'EXPIREINDEX %s from %s accepted (%d seconds)', $name,
