@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 1.64.2.3 $ ; $Date: 2003/06/26 07:54:03 $ 
+# RCS Identication ; $Revision: 1.83.2.1 $ ; $Date: 2004/05/14 08:36:25 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,6 +26,7 @@ package Conf;
 use Log;
 use Language;
 use wwslib;
+use CAS;
 
 require Exporter;
 use Carp;
@@ -36,22 +37,25 @@ use Carp;
 my @valid_options = qw(
 		       avg bounce_warn_rate bounce_halt_rate chk_cert_expiration_task expire_bounce_task
 		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod 
-		       cookie create_list crl_dir crl_update_task db_host db_env db_name db_options db_passwd db_type db_user 
-		       db_port db_additional_subscriber_fields db_additional_user_fields
+		       cookie cookie_cas_expire create_list crl_dir crl_update_task db_host db_env db_name 
+		       db_options db_passwd db_type db_user db_port db_additional_subscriber_fields db_additional_user_fields
 		       default_shared_quota default_archive_quota default_list_priority edit_list email etc
 		       global_remind home host domain lang listmaster log_socket_type log_level 
-		       misaddressed_commands misaddressed_commands_regexp max_size maxsmtp msgcat nrcpt owner_priority pidfile spool queue 
-		       queueauth queuetask queuebounce queuedigest queueexpire queuemod queuesubscribe queueoutgoing tmpdir
+		       misaddressed_commands misaddressed_commands_regexp max_size maxsmtp msgcat nrcpt 
+		       owner_priority pidfile spool queue queueauth queuetask queuebounce queuedigest 
+		       queueexpire queuemod queuesubscribe queueoutgoing tmpdir
 		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor
-		       purge_user_table_task
+		       purge_user_table_task  purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
+		       minimum_bouncing_count minimum_bouncing_period bounce_delay 
+		       default_bounce_level1_rate default_bounce_level2_rate 
 		       remind_return_path request_priority rfc2369_header_fields sendmail sendmail_args sleep 
 		       sort sympa_priority syslog log_smtp umask welcome_return_path wwsympa_url
                        openssl capath cafile  key_passwd ssl_cert_dir remove_headers
 		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields
 		       dark_color light_color text_color bg_color error_color selected_color shaded_color
 		       ldap_export_name ldap_export_host ldap_export_suffix ldap_export_password
-		       ldap_export_dnmanager ldap_export_connection_timeout
-		       list_check_smtp list_check_suffixes  spam_protection web_archive_spam_protection
+		       ldap_export_dnmanager ldap_export_connection_timeout urlize_min_size
+		       list_check_smtp list_check_suffixes  spam_protection web_archive_spam_protection soap_url
 );
 
 my %old_options = ('trusted_ca_options' => 'capath,cafile');
@@ -116,7 +120,7 @@ my %Default_Conf =
      'owner_priority' => 9,
      'lang' => 'us',
      'misaddressed_commands' => 'reject',
-     'misaddressed_commands_regexp' => '(subscribe|unsubscribe|signoff)',
+     'misaddressed_commands_regexp' => '(subscribe|unsubscribe|signoff|set\s+(\S+)\s+(mail|nomail|digest))',
      'max_size' => 5242880,
      'edit_list' => 'owner',
      'create_list' => 'public_listmaster',
@@ -125,6 +129,7 @@ my %Default_Conf =
      'bounce_warn_rate' => '30',
      'bounce_halt_rate' => '50',
      'cookie' => undef,
+     'cookie_cas_expire' => '6',
      'loop_command_max' => 200,
      'loop_command_sampling_delay' => 3600,
      'loop_command_decrease_factor' => 0.5,
@@ -153,13 +158,23 @@ my %Default_Conf =
      'list_check_suffixes' => 'request,owner,editor,unsubscribe,subscribe',
      'expire_bounce_task' => 'daily',
      'purge_user_table_task' => 'monthly',
+     'purge_orphan_bounces_task' => 'monthly',
+     'eval_bouncers_task' => 'daily',
+     'process_bouncers_task' => 'weekly',
      'default_archive_quota' => '',
      'default_shared_quota' => '',
      'capath' => '',
      'cafile' => '',
      'spam_protection' => 'javascript',
-     'web_archive_spam_protection' => 'cookie'
-   );
+     'web_archive_spam_protection' => 'cookie',
+     'minimum_bouncing_count' => 10,
+     'minimum_bouncing_period' => 10,
+     'bounce_delay' => 0,
+     'default_bounce_level1_rate' => 45,
+     'default_bounce_level2_rate' => 75,
+     'soap_url' => '',
+     'urlize_min_size' => 10240 ## 10Kb
+     );
    
 my $wwsconf;
 %Conf = ();
@@ -212,8 +227,9 @@ sub load {
     $o{'domain'} = $o{'host'} if (defined $o{'host'}) ;
     
     unless ( (defined $o{'cafile'}) || (defined $o{'capath'} )) {
-	$o{'cafile'} = '--ETCBINDIR--/ca-bundle.crt';
+	$o{'cafile'}[0] = '--ETCBINDIR--/ca-bundle.crt';
     }   
+
     my $spool = $o{'spool'}[0] || $Default_Conf{'spool'};
 
     unless (defined $o{'queuedigest'}) {
@@ -261,10 +277,10 @@ sub load {
 	$Conf{$i} = $o{$i}[0] || $Default_Conf{$i};
     }
 
-    ## LDAP directories for exportation
-	my @array = &_load_auth();
-	$Conf{'ldap_array'} = [@array];
-	
+    
+    my @array = &_load_auth();
+    $Conf{'auth_services'} = [@array];
+
     if ($Conf{'ldap_export_name'}) {    
 	##Export
 	$Conf{'ldap_export'} = {$Conf{'ldap_export_name'} => { 'host' => $Conf{'ldap_export_host'},
@@ -275,8 +291,7 @@ sub load {
 							   }
 			    };
     }
-    
-    
+        
     my $p = 1;
     foreach (split(/,/, $Conf{'sort'})) {
 	$Conf{'poids'}{$_} = $p++;
@@ -358,7 +373,10 @@ sub load_robots {
 				  list_check_smtp => 1,
 				  list_check_suffixes => 1,
 				  spam_protection => 1,
-				  web_archive_spam_protection => 1
+				  web_archive_spam_protection => 1,
+				  bounce_level1_rate => 1,
+				  bounce_level2_rate => 1,
+				  soap_url => 1,
 				  );
 
     ## Load wwsympa.conf
@@ -371,6 +389,11 @@ sub load_robots {
 	return undef;
     }
 
+    ## Set the defaults based on sympa.conf and wwsympa.conf first
+    foreach my $key (keys %valid_robot_key_words) {
+	$robot_conf->{$Conf{'domain'}}{$key} = $Conf{$key};
+    }
+
     foreach $robot (readdir(DIR)) {
 	next unless (-d "$Conf{'etc'}/$robot");
 	next unless (-r "$Conf{'etc'}/$robot/robot.conf");
@@ -384,7 +407,6 @@ sub load_robots {
 	    if (/^\s*(\S+)\s+(.+)\s*$/io) {
 		my($keyword, $value) = ($1, $2);
 		$value =~ s/\s*$//;
-
 		$keyword = lc($keyword);
 		$value = lc($value) unless ($keyword eq 'title');
 		if ($valid_robot_key_words{$keyword}) {
@@ -414,21 +436,32 @@ sub load_robots {
 	$robot_conf->{$robot}{'sympa'} = $robot_conf->{$robot}{'email'}.'@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'request'} = $robot_conf->{$robot}{'email'}.'-request@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'cookie_domain'} ||= 'localhost';
-	$Conf{'robot_by_http_host'}{$robot_conf->{$robot}{'http_host'}} = $robot ;
+	#$robot_conf->{$robot}{'soap_url'} ||= $Conf{'soap_url'};
+
+	my ($host, $path);
+	if ($robot_conf->{$robot}{'http_host'} =~ /^([^\/]+)(\/.*)$/) {
+	    ($host, $path) = ($1,$2);
+	}else {
+	    ($host, $path) = ($robot_conf->{$robot}{'http_host'}, '/');
+	}
+	$Conf{'robot_by_http_host'}{$host}{$path} = $robot ;
 	
+	## Create a hash to deduce robot from SOAP url
+	if ($robot_conf->{$robot}{'soap_url'}) {
+	    my $url = $robot_conf->{$robot}{'soap_url'};
+	    $url =~ s/^http(s)?:\/\/(.+)$/$2/;
+	    $Conf{'robot_by_soap_url'}{$url} = $robot;
+	}
 
 	close (ROBOT_CONF);
     }
     closedir(DIR);
-
-    ## Add default robot conf
-    ## Missing parameters from wwsympa.conf !!!
-    foreach my $key (keys %valid_robot_key_words) {
-	#if (! defined $Conf{$key}) {
-	#    printf STDERR "OOps: %s\n", $key;
-	#    next;
-	#}
-	$robot_conf->{$Conf{'domain'}}{$key} = $Conf{$key};
+    
+    ## Default SOAP URL corresponds to default robot
+    if ($Conf{'soap_url'}) {
+	my $url = $Conf{'soap_url'};
+	$url =~ s/^http(s)?:\/\/(.+)$/$2/;
+	$Conf{'robot_by_soap_url'}{$url} = $Conf{'domain'};
     }
 
     return ($robot_conf);
@@ -467,6 +500,24 @@ sub checkfiles {
 	    }
 	}
 
+    ## Check cafile and capath access
+    if (defined $Conf{'cafile'} && $Conf{'cafile'}) {
+	unless (-f $Conf{'cafile'} && -r $Conf{'cafile'}) {
+	    &do_log('err', 'Cannot access cafile %s', $Conf{'cafile'});
+	    &List::send_notify_to_listmaster('cannot_access_cafile', $Conf{'domain'}, $Conf{'cafile'});
+	    $config_err++;
+	}
+    }
+
+    if (defined $Conf{'capath'} && $Conf{'capath'}) {
+	unless (-d $Conf{'capath'} && -x $Conf{'capath'}) {
+	    &do_log('err', 'Cannot access capath %s', $Conf{'capath'});
+	    &List::send_notify_to_listmaster('cannot_access_capath', $Conf{'domain'}, $Conf{'capath'});
+	    $config_err++;
+	}
+    }
+
+
     return undef if ($config_err);
     return 1;
 }
@@ -475,57 +526,159 @@ sub checkfiles {
 ##########################################
 
 sub _load_auth {
-    my $config = $Conf{'etc'}.'/auth.conf';
+    
+    my $config;
+    unless ($config = &tools::get_filename('etc', 'auth.conf', $Conf{'domain'})) {
+	do_log('err',"_load_auth: Unable to find auth.conf");
+	return undef;
+    }
+
     my $line_num = 0;
     my $config_err = 0;
     my @paragraphs;
-    my $current_paragraph = {};
+    my %result;
+    my $current_paragraph ;
+
+    my %valid_keywords = ('ldap' => {'regexp' => '.*',
+				     'negative_regexp' => '.*',
+				     'host' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+				     'timeout' => '\d+',
+				     'suffix' => '.+',
+				     'bind_dn' => '.+',
+				     'bind_password' => '.+',
+				     'get_dn_by_uid_filter' => '.+',
+				     'get_dn_by_email_filter' => '.+',
+				     'email_attribute' => '\w+',
+				     'alternative_email_attribute' => '(\w+)(,\w+)*',
+				     'scope' => 'base|one|sub',
+				     'authentication_info_url' => 'http(s)?:/.*',
+				     'use_ssl' => '1',
+				     'ssl_version' => 'sslv2/3|sslv2|sslv3|tlsv1',
+				     'ssl_ciphers' => '[\w:]+' },
+			  
+			  'user_table' => {'regexp' => '.*',
+					   'negative_regexp' => '.*'},
+			  
+			  'cas' => {'base_url' => 'http(s)?:/.*',
+				    'non_blocking_redirection' => 'on|off',
+				    'login_path' => '.*',
+				    'logout_path' => '.*',
+				    'service_validate_path' => '.*',
+				    'proxy_path' => '.*',
+				    'proxy_validate_path' => '.*',
+				    'auth_service_name' => '.*',
+				    'authentication_info_url' => 'http(s)?:/.*',
+				    'ldap_host' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+				    'ldap_bind_dn' => '.+',
+				    'ldap_bind_password' => '.+',
+				    'ldap_timeout'=> '\d+',
+				    'ldap_suffix'=> '.+',
+				    'ldap_scope' => 'base|one|sub',
+				    'ldap_get_email_by_uid_filter' => '.+',
+				    'ldap_email_attribute' => '\w+',
+				    'ldap_use_ssl' => '1',
+				    'ldap_ssl_version' => 'sslv2/3|sslv2|sslv3|tlsv1',
+				    'ldap_ssl_ciphers' => '[\w:]+'
+				    },
+			  'generic_sso' => {'service_name' => '.+',
+					    'service_id' => '\S+',
+					    'http_header_prefix' => '\w+',
+					    'email_http_header' => '\w+'
+					    }
+			  );
+    
+
 
     ## Open the configuration file or return and read the lines.
     unless (open(IN, $config)) {
-	do_log('notice',"load: Unable to open %s: %s\n", $config, $!);
+	do_log('notice',"_load_auth: Unable to open %s: %s", $config, $!);
 	return undef;
     }
     
+    $Conf{'cas_number'} = 0;
+    $Conf{'generic_sso_number'} = 0;
+    $Conf{'ldap_number'} = 0;
+    $Conf{'use_passwd'} = 0;
+    
     ## Parsing  auth.conf
     while (<IN>) {
- 	
+
 	$line_num++;
-	next if (/^\s*[\#\;]/o);	
-	
-      	if (/^\s*(\S+)\s*$/o) {$parag_name = $1;}
-	
-	if (/^\s*(\S+)\s+(.*\S)\s*$/o){
-	    
+	next if (/^\s*[\#\;]/o);		
+
+	if (/^\s*(ldap|cas|user_table|generic_sso)\s*$/io) {
+	    $current_paragraph->{'auth_type'} = lc($1);
+	}elsif (/^\s*(\S+)\s+(.*\S)\s*$/o){
 	    my ($keyword,$value) = ($1,$2);
+	    unless (defined $valid_keywords{$current_paragraph->{'auth_type'}}{$keyword}) {
+		do_log('notice',"_load_auth: unknown keyword '%s' in %s line %d", $keyword, $config, $line_num);
+		next;
+	    }
+	    unless ($value =~ /^$valid_keywords{$current_paragraph->{'auth_type'}}{$keyword}$/) {
+		do_log('notice',"_load_auth: unknown format '%s' for keyword '%s' in %s line %d", $value, $keyword, $config,$line_num);
+		next;
+	    }
+	    
 	    $current_paragraph->{$keyword} = $value;
 	}
-	
-	if (defined($current_paragraph) && (/^\s+$/o)) {
-	    push(@paragraphs,$current_paragraph);
-	    undef $current_paragraph;
+
+	## process current paragraph
+	if (/^\s+$/o || eof(IN)) {
+	    if (defined($current_paragraph)) {
+		
+		if ($current_paragraph->{'auth_type'} eq 'cas') {
+		    unless (defined $current_paragraph->{'base_url'}) {
+			&do_log('err','Incorrect CAS paragraph in auth.conf');
+			next;
+		    }
+
+		    my %cas_param = (casUrl => $current_paragraph->{'base_url'});
+
+		    ## Optional parameters
+		    ## We should also cope with X509 CAs
+		    $cas_param->{'loginPath'} = $current_paragraph->{'login_path'} 
+		    if (defined $current_paragraph->{'login_path'});
+		    $cas_param->{'logoutPath'} = $current_paragraph->{'logout_path'} 
+		    if (defined $current_paragraph->{'logout_path'});
+		    $cas_param->{'serviceValidatePath'} = $current_paragraph->{'service_validate_path'} 
+		    if (defined $current_paragraph->{'service_validate_path'});
+		    $cas_param->{'proxyPath'} = $current_paragraph->{'proxy_path'} 
+		    if (defined $current_paragraph->{'proxy_path'});
+		    $cas_param->{'proxyValidatePath'} = $current_paragraph->{'proxy_validate_path'} 
+		    if (defined $current_paragraph->{'proxy_validate_path'});
+		    
+		    $current_paragraph->{'cas_server'} = new CAS(%cas_param);
+		    unless (defined $current_paragraph->{'cas_server'}) {
+			&do_log('err', 'Failed to create CAS object for %s : %s', 
+				$current_paragraph->{'base_url'}, &CAS::get_errors());
+			next;
+		    }
+
+		    $Conf{'cas_number'}  ++ ;
+		    $Conf{'cas_id'}{$current_paragraph->{'auth_service_name'}} =  $#paragraphs+1 ; 
+		}elsif($current_paragraph->{'auth_type'} eq 'generic_sso') {
+		    $Conf{'generic_sso_number'}  ++ ;
+		    $Conf{'generic_sso_id'}{$current_paragraph->{'service_id'}} =  $#paragraphs+1 ; 
+		}elsif($current_paragraph->{'auth_type'} eq 'ldap') {
+		    $Conf{'ldap'}  ++ ;
+		    $Conf{'use_passwd'} = 1;
+		}elsif($current_paragraph->{'auth_type'} eq 'user_table') {
+		    $Conf{'use_passwd'} = 1;
+		}
+		# setting default
+		$current_paragraph->{'regexp'} = '.*' unless (defined($current_paragraph->{'regexp'})) ;
+		$current_paragraph->{'non_blocking_redirection'} = 'on' unless (defined($current_paragraph->{'non_blocking_redirection'})) ;
+		push(@paragraphs,$current_paragraph);
+		
+		undef $current_paragraph;
+	    } 
+	    next ;
 	}
     }
+    close(IN); 
 
-    if (defined($current_paragraph)) {
-	push(@paragraphs,$current_paragraph);
-	undef $current_paragraph;
-    }
+    return @paragraphs;
     
-  		   	
-    close(IN);
-
-    my @auth;
-    ## Do we have all required values ?
-    for $i ( 0 .. $#paragraphs ) {
-	for $key (sort keys %{ $paragraphs[$i] }) {
-	    $auth[$i]{$key} = $paragraphs[$i]{$key};
-	}
-    }
-
-return @auth;
-
-
 }
 
 ## returns a robot conf parameter
