@@ -1,5 +1,5 @@
 /* boucequeue.c - This program does the non-delivery deports spooling
-  RCS Identication ; $Revision: 1.3 $ ; $Date: 2001/12/04 13:19:03 $ 
+  RCS Identication ; $Revision: 1.4 $ ; $Date: 2003/10/07 12:04:47 $ 
 
   Sympa - SYsteme de Multi-Postage Automatique
   Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -84,7 +84,7 @@ main(int argn, char **argv)
 
    /* Usage : bouncequeue list-name */
    if (argn != 2) {
-      exit(EX_TEMPFAIL);
+      exit(EX_USAGE);
    }
 
    listname = malloc(strlen(argv[1]) + 1);
@@ -92,9 +92,9 @@ main(int argn, char **argv)
      strcpy(listname, argv[1]);
 
    if ((bouncedir = readconf(CONFIG)) == NULL)
-      exit(EX_TEMPFAIL);
+      exit(EX_CONFIG);
    if (chdir(bouncedir) == -1) {
-      exit(EX_TEMPFAIL);
+      exit(EX_NOPERM);
    }
    umask(027);
    sprintf(qfile, "T.%s.%ld.%d", listname, time(NULL), getpid());
