@@ -1,5 +1,5 @@
 /* queue.c - This program does the messages spooling
-   RCS Identication ; $Revision: 1.3 $ ; $Date: 2001/12/04 13:19:03 $ 
+   RCS Identication ; $Revision: 1.4 $ ; $Date: 2003/10/07 12:04:47 $ 
 
    Sympa - SYsteme de Multi-Postage Automatique
    Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,7 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-static char rcsid[] = "(@)$Id: queue.c,v 1.3 2001/12/04 13:19:03 aumont Exp $";
+static char rcsid[] = "(@)$Id: queue.c,v 1.4 2003/10/07 12:04:47 salaun Exp $";
 
 static char     qfile[128];
 static char     buf[16384];
@@ -88,7 +88,7 @@ main(int argn, char **argv)
    if ((argn < 2) || (argn >3)) {
      fprintf(stderr,"%s: usage error, one one list-name argument expected.\n",
             argv[0]);
-      exit(EX_TEMPFAIL);
+      exit(EX_USAGE);
    }
 
    if (argn == 2) {
@@ -107,13 +107,13 @@ main(int argn, char **argv)
    if ((queuedir = readconf(CONFIG)) == NULL){
      fprintf(stderr,"%s: cannot read configuration file '%s'.\n",
             argv[0],CONFIG);
-      exit(EX_TEMPFAIL);
+      exit(EX_CONFIG);
    }
    if (chdir(queuedir) == -1) {
      char* buffer=(char*)malloc(strlen(argv[0])+strlen(queuedir)+80);
      sprintf(buffer,"%s: while changing dir to '%s'",argv[0],queuedir);
      perror(buffer);
-     exit(EX_TEMPFAIL);
+     exit(EX_NOPERM);
    }
    umask(027);
    sprintf(qfile, "T.%s.%ld.%d", listname, time(NULL), getpid());
