@@ -1,5 +1,5 @@
 # tools.pl - This module provides various tools for Sympa
-# RCS Identication ; $Revision: 1.90.2.10 $ ; $Date: 2003/07/17 07:47:48 $ 
+# RCS Identication ; $Revision: 1.130.2.3 $ ; $Date: 2004/06/03 15:03:38 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -28,13 +28,33 @@ use Conf;
 use Language;
 use Log;
 use Time::Local;
+use File::Find;
+use Digest::MD5;
 
 ## RCS identification.
-#my $id = '@(#)$Id: tools.pl,v 1.90.2.10 2003/07/17 07:47:48 salaun Exp $';
+#my $id = '@(#)$Id: tools.pl,v 1.130.2.3 2004/06/03 15:03:38 olivier.salaun Exp $';
 
 ## global var to store a CipherSaber object 
 my $cipher;
 
+## Regexps for list params
+## Caution : if this regexp changes (more/less parenthesis), then regexp using it should 
+## also be changed
+%regexp = ('email' => '([\w\-\_\.\/\+\=\']+|\".*\")\@[\w\-]+(\.[\w\-]+)+',
+	   'host' => '[\w\.\-]+',
+	   'multiple_host_with_port' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+	   'listname' => '[a-z0-9][a-z0-9\-\.\+_]*',
+	   'sql_query' => '(SELECT|select).*',
+	   'scenario' => '[\w,\.\-]+',
+	   'task' => '\w+'
+	   );
+
+my %openssl_errors = (1 => 'an error occurred parsing the command options',
+		      2 => 'one of the input files could not be read',
+		      3 => 'an error occurred creating the PKCS#7 file or when reading the MIME message',
+		      4 => 'an error occurred decrypting or verifying the message',
+		      5 => 'the message was verified correctly but an error occurred writing out the signers certificates');
+
 ## Sorts the list of adresses by domain name
 ## Input : users hash
 ## Sort by domain.
@@ -47,13 +67,13 @@ sub sortbydomain {
 }
 
 ## Safefork does several tries before it gives up.
-## Do 3 trials and wait 10 seconds between each.
+## Do 3 trials and wait 10 seconds * $i between each.
 ## Exit with a fatal error is fork failed after all
 ## tests have been exhausted.
 sub safefork {
    my($i, $pid);
    
-   for ($i = 1; $i < 360; $i++) {
+   for ($i = 1; $i < 4; $i++) {
       my($pid) = fork;
       return $pid if (defined($pid));
       do_log ('warning', "Can't create new process in safefork: %m");
@@ -212,6 +232,93 @@ sub load_create_list_conf {
     return $conf;
 }
 
+## Loads a complete translation index
+sub load_index {
+    my ($lang, $file) = @_;
+    &do_log('debug', 'tools::load_index(%s,%s)', $lang, $file);
+
+    unless (open INDEX, $file) {
+	do_log('err', 'Unable to open file %s: %s', $file, $!);
+	return undef;
+    }
+    
+    my %index; 
+    my $current_tpl;
+    while (<INDEX>) {
+	if (/^file\s+(\w+)\s*$/) {
+	    $current_tpl = $1;
+	    next;
+	}elsif (/^\s*$/) {
+	    $current_tpl = undef;
+	}elsif ($current_tpl && /^(\d+)\s+(.+)$/) {
+	    $index{$current_tpl}{$1} = $2;
+	}
+    }
+    close INDEX;
+
+    return \%index;
+}
+
+## Saves a complete translation index into file
+sub save_index {
+    my ($lang, $file, $data) = @_;
+    &do_log('debug', 'tools::save_index(%s,%s)', $lang, $file);
+
+    unless (open INDEX, ">$file") {
+	do_log('err', 'Unable to write to file %s: %s', $file, $!);
+	return undef;
+    }
+
+    foreach my $tpl (sort keys %{$data}) {
+	printf INDEX "file %s\n", $tpl;
+	foreach my $ref (sort  %{$data->{$tpl}}) {
+	    printf INDEX "%d %s\n", $ref, $data->{$tpl}{$ref};
+	}
+	print "\n";
+    }
+    close INDEX;
+
+    return 1;
+}
+
+## Returns a structure representing a template translation
+sub load_translation {
+    my ($tpl, $lang, $robot) = @_;
+
+    my $index_file =  &get_filename('etc', "wws_templates/index.$lang", $robot);
+    my $src_file =  &get_filename('etc', "wws_templates/$tpl.src", $robot);
+
+    my %references;
+
+    unless (open SOURCE, $src_file) {
+	do_log('err', 'Unable to open file %s: %s', $src_file, $!);
+	return undef;
+    }
+    
+    while (<SOURCE>) {
+	s/{ref(\d+):([^}]+)}/&get_ref($1, $2, \%references)/eg;
+    }
+
+    close SOURCE;
+
+    my $index = &load_index($lang, $index_file);
+
+    my %trans;
+    foreach my $i (keys %references) {
+        $trans{$i} = {'default' => &escape_html($references{$i}),
+	 	      'translation' => &escape_html($index->{$tpl}{$i})};
+    }
+    return \%trans;
+}
+
+## Get data from a translated reference
+sub get_ref {
+    my ($ref, $string, $references) = @_;
+
+    $references->{$ref} = $string;
+    
+}
+
 sub _add_topic {
     my ($name, $title) = @_;
     my $topic = {};
@@ -265,8 +372,7 @@ sub smime_sign {
     do_log('debug2', 'tools::smime_sign (%s,%s)',$in_msg,$list);
 
     my $self = new List($list);
-    my $cert = $self->{'dir'}."/cert.pem";
-    my $key = $self->{'dir'}."/private_key";
+    my($cert, $key) = &smime_find_keys($self->{dir}, 'sign');
     my $temporary_file = $Conf{'tmpdir'}."/".$list.".".$$ ;    
 
     my ($signed_msg,$pass_option );
@@ -319,6 +425,12 @@ sub smime_sign {
     }
     close NEWMSG ;
 
+    my $status = $?/256 ;
+    unless ($status == 0) {
+	do_log('notice', 'Unable to S/MIME sign message : status = %d', $status);
+	return undef;	
+    }
+
     unlink ($temporary_file) unless ($main::options{'debug'}) ;
     
     ## foreach header defined in  the incomming message but undefined in the
@@ -362,12 +474,14 @@ sub smime_sign_check {
 
     unless (open (MSGDUMP, "| $Conf{'openssl'} smime -verify  $trusted_ca_options -signer $temporary_file > /dev/null")) {
 
-	do_log('err', "unabe to verify smime signature from $sender $verify");
+	do_log('err', "unable to verify smime signature from $sender $verify");
 	return undef ;
     }
 
     if ($message->{'smime_crypted'}) {
-	print MSGDUMP ${$message->{'decrypted_msg_as_string'}};
+	$message->{'msg'}->head->print(\*MSGDUMP);
+	print MSGDUMP "\n";
+	print MSGDUMP ${$message->{'msg_as_string'}};
     }else {
 	unless (open MSG, $file) {
 	    do_log('err', 'Unable to open file %s: %s', $file, $!);
@@ -376,41 +490,26 @@ sub smime_sign_check {
 	print MSGDUMP <MSG>;
     }
 
-    close MSGDUMP;
-    my $status = $?/256 ;
     close MSG;
-    if ($status == '1') {
-	do_log('err', "Openssl ERROR while parsing the command options : $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
+    close MSGDUMP;
+
+    my $status = $?/256 ;
+    unless ($status == 0) {
+	do_log('err', 'Unable to check S/MIME signature : %s', $openssl_errors{$status});
 	return undef ;
     }
-    if ($status == '2') {
-	do_log('err', "One of input file could not be read : $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
-	return undef ;
-    }
-    if ($status == '3') {
-	do_log('err', "Error creating the PKCS#7 file: $Conf{'openssl'} smime -verify  $Conf{'trusted_ca_options'} -signer  $temporary_file");
-	return undef ;
-    }
-    if ($status == '4') {
-	do_log('err', "Error while verifying the message signature. The message may have been corrupted");
-	return undef ;
-    }
-    if ($status == '5') {
-	do_log('err', "The message signature is checked but could not write out the signers certificates");
-    }    
     
     ## second step is the message signer match the sender
     ## a better analyse should be performed to extract the signer email. 
-    my $signer = `cat $temporary_file | $Conf{'openssl'}  x509 -subject -noout`;
-    chomp $signer;
+    my $signer = smime_parse_cert({file => $temporary_file});
 
-    unless ($signer =~ /email=$sender/i) {
+    unless ($signer->{'email'} eq lc($sender)) {
 	unlink($temporary_file) unless ($main::options{'debug'}) ;	
-	do_log('notice', "S/MIME signed message, sender($sender) do NOT match signer($signer)",$sender,$signer);
+	&do_log('notice', "S/MIME signed message, sender(%s) does NOT match signer(%s)",$sender,$signer->{'email'});
 	return undef;
     }
 
-    do_log('debug', "S/MIME signed message, signature checked and sender match signer(%s)",$signer);
+    &do_log('debug', "S/MIME signed message, signature checked and sender match signer(%s)",$signer->{'email'});
     ## store the signer certificat
     unless (-d $Conf{'ssl_cert_dir'}) {
 	if ( mkdir ($Conf{'ssl_cert_dir'}, 0775)) {
@@ -419,20 +518,112 @@ sub smime_sign_check {
 	    do_log('err', "Unable to create user certificat directory $Conf{'ssl_cert_dir'}");
 	}
     }
-    my $filename = "$Conf{'ssl_cert_dir'}/".&escape_chars($sender);
-    
-    open (CERTIF,$temporary_file);
-    if (open (USERCERTIF, "> $filename")) {
-	print USERCERTIF <CERTIF> ;
-	close USERCERTIF ;
-    }else{
-	&do_log('err','Unable to rename %s %s',$temporary_file,$filename);
+
+    ## It gets a bit complicated now. openssl smime -signer only puts
+    ## the _signing_ certificate into the given file; to get all included
+    ## certs, we need to extract them from the signature proper, and then
+    ## we need to check if they are for our user (CA and intermediate certs
+    ## are also included), and look at the purpose:
+    ## "S/MIME signing : Yes/No"
+    ## "S/MIME encryption : Yes/No"
+    my $certbundle = "$Conf{tmpdir}/certbundle.$$";
+    my $tmpcert = "$Conf{tmpdir}/cert.$$";
+    my $nparts = $message->{msg}->parts;
+    my $extracted = 0;
+    &do_log('debug2', "smime_sign_check: parsing $nparts parts");
+    if($nparts == 0) { # could be opaque signing...
+	$extracted +=&smime_extract_certs($message->{msg}, $certbundle);
+    } else {
+	for (my $i = 0; $i < $nparts; $i++) {
+	    my $part = $message->{msg}->parts($i);
+	    $extracted += &smime_extract_certs($part, $certbundle);
+	    last if $extracted;
+	}
     }
-    close CERTIF;
     
+    unless($extracted) {
+	&do_log('err', "No application/x-pkcs7-* parts found");
+	return undef;
+    }
+
+    unless(open(BUNDLE, $certbundle)) {
+	&do_log('err', "Can't open cert bundle $certbundle: $!");
+	return undef;
+    }
     
-    unlink($temporary_file) unless ($main::options{'debug'}) ;	
-    
+    ## read it in, split on "-----END CERTIFICATE-----"
+    my $cert = '';
+    my(%certs);
+    while(<BUNDLE>) {
+	$cert .= $_;
+	if(/^-----END CERTIFICATE-----$/) {
+	    my $workcert = $cert;
+	    $cert = '';
+	    unless(open(CERT, ">$tmpcert")) {
+		&do_log('err', "Can't create $tmpcert: $!");
+		return undef;
+	    }
+	    print CERT $workcert;
+	    close(CERT);
+	    my($parsed) = &smime_parse_cert({file => $tmpcert});
+	    unless($parsed) {
+		&do_log('err', 'No result from smime_parse_cert');
+		return undef;
+	    }
+	    unless($parsed->{'email'}) {
+		&do_log('debug', "No email in cert for $parsed->{subject}, skipping");
+		next;
+	    }
+	    
+	    &do_log('debug', "Found cert for <$parsed->{email}>");
+
+	    if (lc($sender) eq $parsed->{email}) {
+		if ($parsed->{'purpose'}{'sign'} && $parsed->{'purpose'}{'enc'}) {
+		    $certs{'both'} = $workcert;
+		    &do_log('debug', 'Found a signing + encryption cert');
+		}elsif ($parsed->{'purpose'}{'sign'}) {
+		    $certs{'sign'} = $workcert;
+		    &do_log('debug', 'Found a signing cert');
+		} elsif($parsed->{'purpose'}{'enc'}) {
+		    $certs{'enc'} = $workcert;
+		    &do_log('debug', 'Found an encryption cert');
+		}
+	    }
+	    last if(($certs{'both'}) || ($certs{'sign'} && $certs{'enc'}));
+	}
+    }
+    close(BUNDLE);
+    if(!($certs{both} || ($certs{sign} && $certs{enc}))) {
+	&do_log('err', "Could not extract certificate for $signer->{email}");
+	return undef;
+    }
+    ## OK, now we have the certs, either a combined sign+encryption one
+    ## or a pair of single-purpose. save them, as email@addr if combined,
+    ## or as email@addr@sign / email@addr@enc for split certs.
+    foreach my $c (keys %certs) {
+	my $fn = "$Conf{ssl_cert_dir}/" . &escape_chars(lc($sender));
+	if ($c ne 'both') {
+	    unlink($fn); # just in case there's an old cert left...
+	    $fn .= "\@$c";
+	}else {
+	    unlink("$fn\@enc");
+	    unlink("$fn\@sign");
+	}
+	&do_log('debug', "Saving $c cert in $fn");
+	unless (open(CERT, ">$fn")) {
+	    &do_log('err', "Unable to create certificate file $fn: $!");
+	    return undef;
+	}
+	print CERT $certs{$c};
+	close(CERT);
+    }
+
+    unless ($main::options{'debug'}) {
+	unlink($temporary_file);
+	unlink($tmpcert);
+	unlink($certbundle);
+    }
+
     $is_signed->{'body'} = 'smime';
     
     # futur version should check if the subject was part of the SMIME signature.
@@ -448,28 +639,51 @@ sub smime_encrypt {
     my $list = shift ;
 
     my $usercert;
+    my $dummy;
     my $cryptedmsg;
     my $encrypted_body;    
 
     &do_log('debug2', 'tools::smime_encrypt( %s, %s', $email, $list);
     if ($list eq 'list') {
 	my $self = new List($email);
-	$usercert = $self->{'dir'}."/cert.pem";
+	($usercert, $dummy) = smime_find_keys($self->{dir}, 'encrypt');
     }else{
-	$usercert = "$Conf{'ssl_cert_dir'}/".&tools::escape_chars($email);
+	my $base = "$Conf{'ssl_cert_dir'}/".&tools::escape_chars($email);
+	if(-f "$base\@enc") {
+	    $usercert = "$base\@enc";
+	} else {
+	    $usercert = "$base";
+	}
     }
     if (-r $usercert) {
 	my $temporary_file = $Conf{'tmpdir'}."/".$email.".".$$ ;
 
 	## encrypt the incomming message parse it.
-        do_log ('debug3', "xxxx $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert");
+        do_log ('debug3', "tools::smime_encrypt : $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert");
+
 	if (!open(MSGDUMP, "| $Conf{'openssl'} smime -encrypt -out $temporary_file -des3 $usercert")) {
 	    &do_log('info', 'Can\'t encrypt message for recipient %s', $email);
 	}
-	$msg_header->print(\*MSGDUMP);
+## don't; cf RFC2633 3.1. netscape 4.7 at least can't parse encrypted stuff
+## that contains a whole header again... since MIME::Tools has got no function
+## for this, we need to manually extract only the MIME headers...
+##	$msg_header->print(\*MSGDUMP);
+##	printf MSGDUMP "\n%s", $msg_body;
+	my $mime_hdr = $msg_header->dup();
+	foreach my $t ($mime_hdr->tags()) {
+	  $mime_hdr->delete($t) unless ($t =~ /^(mime|content)-/i);
+	}
+	$mime_hdr->print(\*MSGDUMP);
+
 	printf MSGDUMP "\n%s", $msg_body;
 	close(MSGDUMP);
 
+	my $status = $?/256 ;
+	unless ($status == 0) {
+	    do_log('err', 'Unable to S/MIME encrypt message : %s', $openssl_errors{$status});
+	    return undef ;
+	}
+
         ## Get as MIME object
 	open (NEWMSG, $temporary_file);
 	my $parser = new MIME::Parser;
@@ -521,17 +735,17 @@ sub smime_decrypt {
     
     &do_log('debug2', 'tools::smime_decrypt message msg from %s,%s',$msg->head->get('from'),$list->{'name'});
 
-    my $certfile = $list->{'dir'}."/cert.pem" ;
-    unless (-r $certfile){
-	do_log('err', "unable to decrypt message : cert missing  $certfile");
+    ## an empty "list" parameter means mail to sympa@, listmaster@...
+    my $dir = $list->{dir};
+    unless ($dir) {
+	$dir = $Conf{home} . '/sympa';
+    }
+    my ($certs,$keys) = smime_find_keys($dir, 'decrypt');
+    unless (defined $certs && @$certs) {
+	do_log('err', "Unable to decrypt message : missing certificate file");
 	return undef;
     }
-    my $keyfile = $list->{'dir'}."/private_key";
 
-    unless (open (MSGDUMP , "> $Conf{'tmpdir'}/MSG.$$")) {
-	&do_log('err', 'unable to open %s/MSG.%s',$Conf{'tmpdir'},$$);
-	return undef;
-    }
     my $temporary_file = $Conf{'tmpdir'}."/".$list->{'name'}.".".$$ ;
     
     ## dump the incomming message.
@@ -540,49 +754,58 @@ sub smime_decrypt {
     }
     $msg->print(\*MSGDUMP);
     close(MSGDUMP);
-
     
-    my $decryptedmsg,$pass_option;
+    my ($decryptedmsg, $pass_option, $msg_as_string);
     if ($Conf{'key_passwd'} ne '') {
 	# if password is define in sympa.conf pass the password to OpenSSL using
 	$pass_option = "-passin file:$Conf{'tmpdir'}/pass.$$";	
     }
 
-    if ($Conf{'key_passwd'} ne '') {
-	unless (&POSIX::mkfifo("$Conf{'tmpdir'}/pass.$$",0600)) {
-	    do_log('notice', 'Unable to make fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
-	    return undef;
+    ## try all keys/certs until one decrypts.
+    while (my $certfile = shift @$certs) {
+	my $keyfile = shift @$keys;
+	&do_log('debug', "Trying decrypt with $certfile, $keyfile");
+	if ($Conf{'key_passwd'} ne '') {
+	    unless (&POSIX::mkfifo("$Conf{'tmpdir'}/pass.$$",0600)) {
+		&do_log('err', 'Unable to make fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+		return undef;
+	    }
 	}
-    }
 
-    open (NEWMSG, "$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option |");
+	&do_log('debug',"$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option");
+	open (NEWMSG, "$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option |");
 
-    if ($Conf{'key_passwd'} ne '') {
-	unless (open (FIFO,"> $Conf{'tmpdir'}/pass.$$")) {
-	    do_log('notice', 'Unable to open fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
-	    return undef;
+	if ($Conf{'key_passwd'} ne '') {
+	    unless (open (FIFO,"> $Conf{'tmpdir'}/pass.$$")) {
+		&do_log('notice', 'Unable to open fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+		return undef;
+	    }
+	    print FIFO $Conf{'key_passwd'};
+	    close FIFO;
+	    unlink ("$Conf{'tmpdir'}/pass.$$");
+	}
+	
+	while (<NEWMSG>) {
+	    $msg_as_string .= $_;
+	}
+	close NEWMSG ;
+	my $status = $?/256;
+	
+	unless ($status == 0) {
+	    do_log('notice', 'Unable to decrypt S/MIME message : %s', $openssl_errors{$status});
+	    next;
+	}
+	
+	unlink ($temporary_file) unless ($main::options{'debug'}) ;
+	
+	my $parser = new MIME::Parser;
+	$parser->output_to_core(1);
+	unless ($decryptedmsg = $parser->parse_data($msg_as_string)) {
+	    &do_log('notice', 'Unable to parse message');
+	    last;
 	}
-	print FIFO $Conf{'key_passwd'};
-	close FIFO;
-	unlink ("$Conf{'tmpdir'}/pass.$$");
-    }
-    
-    my $msg_as_string;
-    while (<NEWMSG>) {
-	$msg_as_string .= $_;
     }
 	
-    close NEWMSG ;
-    unlink ($temporary_file) unless ($main::options{'debug'}) ;
-    
-    ## Create a MIME object with decrypted message
-    my $parser = new MIME::Parser;
-    $parser->output_to_core(1);
-    unless ($decryptedmsg = $parser->parse_data($msg_as_string)) {
-	do_log('notice', 'Unable to parse message');
-	return undef;
-    }
-
     ## Now remove headers from $msg_as_string
     my @msg_tab = split(/\n/, $msg_as_string);
     my $line;
@@ -629,7 +852,7 @@ sub as_singlepart {
     }
 
     # Now look for the preferred_type and if found, make this the main message
-    my @parts = $msg->parts();
+    @parts = $msg->parts();
     foreach my $index (0..$#parts) {
 	if ($parts[$index]->effective_type() =~ /^$preferred_type$/) {
 	    ## Only keep the first matching part
@@ -653,7 +876,7 @@ sub escape_chars {
     ## Escape chars
     ##  !"#$%&'()+,:;<=>?[] AND accented chars
     ## escape % first
-    foreach my $i (0x25,0x20..0x24,0x26..0x2c,0x3a..0x3f,0x5b,0x5d,0xa0..0xff) {
+    foreach my $i (0x25,0x20..0x24,0x26..0x2c,0x3a..0x3f,0x5b,0x5d,0x80..0x9f,0xa0..0xff) {
 	next if ($i == $ord_except);
 	my $hex_i = sprintf "%lx", $i;
 	$s =~ s/\x$hex_i/%$hex_i/g;
@@ -668,7 +891,7 @@ sub unescape_chars {
     my $s = shift;
 
     $s =~ s/%a5/\//g;  ## Special traetment for '/'
-    foreach my $i (0x20..0x2c,0x3a..0x3f,0x5b,0x5d,0xa0..0xff) {
+    foreach my $i (0x20..0x2c,0x3a..0x3f,0x5b,0x5d,0x80..0x9f,0xa0..0xff) {
 	my $hex_i = sprintf "%lx", $i;
 	my $hex_s = sprintf "%c", $i;
 	$s =~ s/%$hex_i/$hex_s/g;
@@ -681,7 +904,9 @@ sub escape_html {
     my $s = shift;
 
     $s =~ s/\"/\&quot\;/g;
-
+    $s =~ s/\</&lt\;/g;
+    $s =~ s/\>/&gt\;/g;
+    
     return $s;
 }
 
@@ -829,7 +1054,6 @@ sub split_mail {
     my $message = shift ; 
     my $pathname = shift ;
     my $dir = shift ;
-    &do_log('debug2', 'tools::split_mail()');
 
     my $head = $message->head ;
     my $body = $message->body ;
@@ -845,10 +1069,10 @@ sub split_mail {
     else { 
 	    my $fileExt ;
 
-	    if ($head->mime_attr("content_type.name") =~ /\.(\S+)\s*\"*$/) {
+	    if ($head->mime_attr("content_type.name") =~ /\.(\w+)\s*\"*$/) {
 		$fileExt = $1 ;
 	    }
-	    elsif ($head->recommended_filename =~ /\.(\S+)\s*\"*$/) {
+	    elsif ($head->recommended_filename =~ /\.(\w+)\s*\"*$/) {
 		$fileExt = $1 ;
 	    }
 	    else {
@@ -862,7 +1086,7 @@ sub split_mail {
 
 	    ## Store body in file 
 	    unless (open OFILE, ">$dir/$pathname.$fileExt") {
-		&do_log('err', "Unable to open $dir/$pathname.$fileExt") ;
+		&do_log('err', "Unable to create $dir/$pathname.$fileExt : $!") ;
 		return undef ; 
 	    }
 	    
@@ -879,6 +1103,7 @@ sub split_mail {
 		    return undef;
 		}
 		$decoder->decode(\*BODY, \*OFILE);
+		close BODY;
 		unlink "$dir/$pathname.$fileExt.$encoding";
 	    }else {
 		$message->print_body (\*OFILE) ;
@@ -889,6 +1114,7 @@ sub split_mail {
 	    ## Delete files created twice or more (with Content-Type.name and Content-Disposition.filename)
 	    $message->purge ;	
     }
+    
     return 1;
 }
 
@@ -1154,10 +1380,10 @@ sub epoch_conv {
     my $duration;
     my $op;
 
-    if ($arg =~ /\+|\-/) {
-	$date = $`;
-	$duration = $';
-	$op = $&;
+    if ($arg =~ /^(.+)(\+|\-)(.+)$/) {
+	$date = $1;
+	$duration = $3;
+	$op = $2;
     } else {
 	$date = $arg;
 	$duration = '';
@@ -1268,15 +1494,16 @@ sub get_filename {
 		}
 	    }
 
-	    ## No 'templates' subdir in list directory
+	    ## Also look for templates in the list's directory
 	    if ($name =~ /^templates\/(.*)$/) {
+		unshift @try, $list->{'dir'}.'/'.$name;
 		unshift @try, $list->{'dir'}.'/'.$1;
 	    }else {
 		unshift @try, $list->{'dir'}.'/'.$name;
 	    }
 	}	
 	foreach my $f (@try) {
-	    &do_log('debug3','get_filname : NAME: %s ; DIR %s', $name, $dir);
+	    &do_log('debug3','get_filname : NAME: %s ; DIR %s', $name, $f  );
 	    if (-r $f) {
 		return $f;
 	    }
@@ -1308,7 +1535,7 @@ sub write_pid {
 	 fatal_err("Could not open %s, exiting", $pidfile);
     } 
     unless (flock(LOCK, 6)) {
-	fatal_err("Could not lock %s: process is probably already running.", $pidfile);
+	fatal_err("Could not lock %s, process is probably already running : %s", $pidfile, $!);
     }
     unless (open(LCK, "> $pidfile")) {
 	fatal_err("Could not open %s, exiting", $pidfile);
@@ -1328,24 +1555,11 @@ sub write_pid {
 sub get_message_id {
     my $robot = shift;
 
-    my $id = sprintf '<sympa.%d.%d.%d@%s>', time, $$, int(rand(999)), $robot;
+    my $id = sprintf 'sympa.%d.%d.%d@%s', time, $$, int(rand(999)), $robot;
 
     return $id;
 }
 
-## Clean email address
-sub clean_email {
-    my $email = shift;
-
-    ## Lower-case
-    $email = lc($email);
-
-    ## remove leading and trailing spaces
-    $email =~ s/^\s*//;
-    $email =~ s/\s*$//;
-
-    return $email;
-}
 
 sub get_dir_size {
     my $dir =shift;
@@ -1372,7 +1586,54 @@ sub get_dir_size {
 sub valid_email {
     my $email = shift;
     
-    $email =~ /^([\w\-\_\.\/\+\=\']+|\".*\")\@[\w\-]+(\.[\w\-]+)+$/;
+    $email =~ /^$tools::regexp{'email'}$/;
+}
+
+## Clean email address
+sub clean_email {
+    my $email = shift;
+
+    ## Lower-case
+    $email = lc($email);
+
+    ## remove leading and trailing spaces
+    $email =~ s/^\s*//;
+    $email =~ s/\s*$//;
+
+    return $email;
+}
+
+## Function for Removing a non-empty directory
+## It takes a variale number of arguments : 
+## it can be a list of directory
+## or few direcoty paths
+sub remove_dir {
+    
+    do_log('info','remove_dir()');
+    
+    foreach my $current_dir (@_){
+	my @tree = split /\//, $current_dir ;
+	if ($#tree < 4) {
+	    do_log('info',"$current_dir not removed (not enough / in directory name)");
+	    next;
+	}
+	finddepth({wanted => \&del, no_chdir => 1},$current_dir);
+    }
+    sub del {
+	my $name = $File::Find::name;
+
+	if (!-l && -d _) {
+	    &do_log('info','removing dir %s',$name);
+	    unless (rmdir($name)) {
+		&do_log('info','Error while removing dir %s',$name);
+	    }
+	}else{
+	    unless (unlink($name)) {
+		&do_log('info','Error while removing file  %s',$name);
+	    }
+	}
+    }
+    return 1;
 }
 
 ## Return canonical email address (lower-cased + space cleanup)
@@ -1389,5 +1650,229 @@ sub get_canonical_email {
     return $email;
 }
 
+## find the appropriate S/MIME keys/certs for $oper in $dir.
+## $oper can be:
+## 'sign' -> return the preferred signing key/cert
+## 'decrypt' -> return a list of possible decryption keys/certs
+## 'encrypt' -> return the preferred encryption key/cert
+## returns ($certs, $keys)
+## for 'sign' and 'encrypt', these are strings containing the absolute filename
+## for 'decrypt', these are arrayrefs containing absolute filenames
+sub smime_find_keys {
+    my($dir, $oper) = @_;
+    &do_log('debug', 'tools::smime_find_keys(%s, %s)', $dir, $oper);
+
+    my(%certs, %keys);
+    my $ext = ($oper eq 'sign' ? 'sign' : 'enc');
+
+    unless (opendir(D, $dir)) {
+	do_log('err', "unable to opendir $dir: $!");
+	return undef;
+    }
+
+    while (my $fn = readdir(D)) {
+	if ($fn =~ /^cert\.pem/) {
+	    $certs{"$dir/$fn"} = 1;
+	}elsif ($fn =~ /^private_key/) {
+	    $keys{"$dir/$fn"} = 1;
+	}
+    }
+    closedir(D);
+
+    foreach my $c (keys %certs) {
+	my $k = $c;
+	$k =~ s/\/cert\.pem/\/private_key/;
+	unless ($keys{$k}) {
+	    do_log('notice', "$c exists, but matching $k doesn't");
+	    delete $certs{$c};
+	}
+    }
+
+    foreach my $k (keys %keys) {
+	my $c = $k;
+	$c =~ s/\/private_key/\/cert\.pem/;
+	unless ($certs{$c}) {
+	    do_log('notice', "$k exists, but matching $c doesn't");
+	    delete $keys{$k};
+	}
+    }
+
+    my ($certs, $keys);
+    if ($oper eq 'decrypt') {
+	$certs = [ sort keys %certs ];
+	$keys = [ sort keys %keys ];
+    }else {
+	if($certs{"$dir/cert.pem.$ext"}) {
+	    $certs = "$dir/cert.pem.$ext";
+	    $keys = "$dir/private_key.$ext";
+	} elsif($certs{"$dir/cert.pem"}) {
+	    $certs = "$dir/cert.pem";
+	    $keys = "$dir/private_key";
+	} else {
+	    do_log('info', "$dir: no certs/keys found for $oper");
+	    return undef;
+	}
+    }
+
+    return ($certs,$keys);
+}
+
+# IN: hashref:
+# file => filename
+# text => PEM-encoded cert
+# OUT: hashref
+# email => email address from cert
+# subject => distinguished name
+# purpose => hashref
+#  enc => true if v3 purpose is encryption
+#  sign => true if v3 purpose is signing
+sub smime_parse_cert {
+    my($arg) = @_;
+    &do_log('debug', 'tools::smime_parse_cert(%s)', join('/',%{$arg}));
+
+    unless (ref($arg)) {
+	&do_log('err', "smime_parse_cert: must be called with hashref, not %s", ref($arg));
+	return undef;
+    }
+
+    ## Load certificate
+    my @cert;
+    if($arg->{'text'}) {
+	@cert = ($arg->{'text'});
+    }elsif ($arg->{file}) {
+	unless (open(PSC, "$arg->{file}")) {
+	    &do_log('err', "smime_parse_cert: open $file: $!");
+	    return undef;
+	}
+	@cert = <PSC>;
+	close(PSC);
+    }else {
+	&do_log('err', 'smime_parse_cert: neither "text" nor "file" given');
+	return undef;
+    }
+
+    ## Extract information from cert
+    my ($tmpfile) = $Conf{'tmpdir'}."/parse_cert.$$";
+    unless (open(PSC, "| $Conf{openssl} x509 -email -subject -purpose -noout > $tmpfile")) {
+	&do_log('err', "smime_parse_cert: open |openssl: $!");
+	return undef;
+    }
+    print PSC join('', @cert);
+
+    unless (close(PSC)) {
+	&do_log('err', "smime_parse_cert: close openssl: $!, $@");
+	return undef;
+    }
+
+    unless (open(PSC, "$tmpfile")) {
+	&do_log('err', "smime_parse_cert: open $tmpfile: $!");
+	return undef;
+    }
+
+    my @output = <PSC>;
+    my %res;
+
+    $res{'email'} = lc($output[0]);
+    chomp $res{'email'};
+
+    if ($output[1] =~ /^subject=\s+(\S.+)\s*$/) {
+	$res{'subject'} = $1;
+    }
+
+    if ($output[2] =~ /^Certificate purposes:/) {
+	foreach my $i (3..$#output) {
+	    $_ = $output[$i];
+	    
+	    if (/^S\/MIME signing : (\S+)/) {
+		$res{purpose}->{sign} = ($1 eq 'Yes');
+	    }elsif (/^S\/MIME encryption : (\S+)/) {
+		$res{purpose}->{enc} = ($1 eq 'Yes');
+	    }
+	}
+    }
+    
+    ## OK, so there's CAs which put the email in the subjectAlternateName only
+    ## and ones that put it in the DN only...
+    if(!$res{email} && ($res{subject} =~ /\/email(address)?=([^\/]+)/)) {
+	$res{email} = $1;
+    }
+    close(PSC);
+    unlink($tmpfile);
+    return \%res;
+}
+
+sub smime_extract_certs {
+    my($mime, $outfile) = @_;
+    &do_log('debug2', "tools::smime_extract_certs(%s)",$mime->mime_type);
+
+    if ($mime->mime_type =~ /application\/(x-)?pkcs7-/) {
+	unless (open(MSGDUMP, "| $Conf{openssl} pkcs7 -print_certs ".
+		     "-inform der > $outfile")) {
+	    &do_log('err', "unable to run openssl pkcs7: $!");
+	    return 0;
+	}
+	print MSGDUMP $mime->bodyhandle->as_string;
+	close(MSGDUMP);
+	if ($?) {
+	    &do_log('err', "openssl pkcs7 returned an error: ", $?/256);
+	    return 0;
+	}
+	return 1;
+    }
+}
+
+## Dump a variable's content
+sub dump_var {
+    my ($var, $level, $fd) = @_;
+    
+    if (ref($var)) {
+	if (ref($var) eq 'ARRAY') {
+	    foreach my $index (0..$#{$var}) {
+		print $fd "\t"x$level.$index."\n";
+		&dump_var($var->[$index], $level+1, $fd);
+	    }
+	}elsif (ref($var) eq 'HASH') {
+	    foreach my $key (sort keys %{$var}) {
+		print $fd "\t"x$level.'_'.$key.'_'."\n";
+		&dump_var($var->{$key}, $level+1, $fd);
+	    }    
+	}
+    }else {
+	if (defined $var) {
+	    print $fd "\t"x$level."'$var'"."\n";
+	}else {
+	    print $fd "\t"x$level."UNDEF\n";
+	}
+    }
+}
+
+## Change X-Sympa-To: header field in the message
+sub change_x_sympa_to {
+    my ($file, $value) = @_;
+    
+    ## Change X-Sympa-To
+    unless (open FILE, $file) {
+	&wwslog('err', "Unable to open '%s' : %s", $file, $!);
+	next;
+    }	 
+    my @content = <FILE>;
+    close FILE;
+    
+    unless (open FILE, ">$file") {
+	&wwslog('err', "Unable to open '%s' : %s", "$file", $!);
+	next;
+    }	 
+    foreach (@content) {
+	if (/^X-Sympa-To:/i) {
+	    $_ = "X-Sympa-To: $value\n";
+	}
+	print FILE;
+    }
+    close FILE;
+    
+    return 1;
+}
+
+
 1;
 
