@@ -1,5 +1,5 @@
 # cookielib.pm - This module includes functions managing HTTP cookies in Sympa
-# RCS Identication ; $Revision: 1.23 $ ; $Date: 2002/06/26 12:28:20 $ 
+# RCS Identication ; $Revision: 1.28 $ ; $Date: 2003/10/31 08:00:58 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -48,12 +48,14 @@ sub check_cookie {
 	
 	next unless ($cookie->name =~ /^(sympauser|user)$/);
 
-	if ($cookie->value =~ /^(.*):(\S+)\s*$/) {
-	    my ($email, $mac) = ($1, $2);
+	my @values = split /:/,$cookie->value; 
+	if ($#values >= 1) {
+	    my ($email, $mac, $auth) = @values;
+	    $auth ||= 'classic';
 
 	    ## Check the MAC
 	    if (&get_mac($email,$secret) eq $mac) {
-		return $email;
+		return ($email, $auth);
 	    }
 	}	
     }
@@ -96,7 +98,7 @@ sub check_lang_cookie {
 
 ## Set user $email cookie, ckecksum use $secret, expire=(now|session|#sec) domain=(localhost|<a domain>)
 sub set_cookie {
-    my ($email, $secret, $http_domain, $expires) = @_ ;
+    my ($email, $secret, $http_domain, $expires, $auth) = @_ ;
 
     unless ($email) {
 	return undef;
@@ -115,6 +117,9 @@ sub set_cookie {
     }
 
     my $value = sprintf '%s:%s', $email, &get_mac($email,$secret);
+    if ($auth ne 'classic') {
+	$value .= ':'.$auth;
+    }
     my $cookie;
     if ($expires =~ /session/i) {
 	$cookie = new CGI::Cookie (-name    => 'sympauser',
@@ -154,10 +159,15 @@ sub set_arc_cookie {
     return 1;
 }
     
+    
 ## Set cookie with lang pref
 sub set_lang_cookie {
     my ($lang,$domain) = @_;
 
+    if ($domain eq 'localhost') {
+	$domain="";
+    }
+
     my $cookie = new CGI::Cookie (-name    => 'sympalang',
 				  -value   => $lang,
 				  -expires => '+1M',
@@ -171,18 +181,45 @@ sub set_lang_cookie {
     return 1;
 }
     
+## Sets an HTTP cookie to be sent to a SOAP client
+sub set_cookie_soap {
+    my ($email,$secret,$http_domain,$expire) = @_ ;
+    my $cookie;
+    my $value;
+    &do_log('debug', 'cookielib::set_cookie_soap(%s,%s,%s,%s)', $email, $secret, $http_domain, $expire);
+
+    # WARNING : to check the cookie the SOAP services does not gives
+    # all the cookie, only it's value so we need ':'
+    $value = sprintf '%s:%s',$email,&get_mac($email,$secret);
+  
+    ## With set-cookie2 max-age of 0 means removing the cookie
+    ## Maximum cookie lifetime is the session
+    $expire ||= 600; ## 10 minutes
+
+    if ($http_domain eq 'localhost') {
+	$cookie = sprintf "%s=%s; Path=/; Max-Age=%s", 'sympauser', $value, $expire;
+    }else {
+	$cookie = sprintf "%s=%s; Domain=%s; Path=/; Max-Age=%s", 'sympauser', $value, $http_domain, $expire;;
+    }
+
+    $ENV{'SOAP_COOKIE_sympauser'} = $cookie;
+
+    ## Send cookie to the client
+    return 1;
+}
+
 ## returns Message Authentication Check code
 sub get_mac {
         my $email = shift ;
 	my $secret = shift ;	
-	&main::wwslog('debug4', "get_mac($email, $secret)");
+	&do_log('debug4', "get_mac($email, $secret)");
 
 	unless ($secret) {
-	    &main::wwslog('err', 'get_mac : failure missing server secret for cookie MD5 digest');
+	    &do_log('err', 'get_mac : failure missing server secret for cookie MD5 digest');
 	    return undef;
 	}
 	unless ($email) {
-	    &main::wwslog('err', 'get_mac : failure missing email adresse or cookie MD5 digest');
+	    &do_log('err', 'get_mac : failure missing email adresse or cookie MD5 digest');
 	    return undef;
 	}
 
@@ -208,7 +245,7 @@ sub check_cookie_extern {
 	
 	next unless ($cookie->name =~ /sympa_altemails/);
 	
-	if ($cookie->value =~ /^(\S+)&(\w+)$/) {
+ 	if ($cookie->value =~ /^(\S+)&(\w+)$/) {
 	    return undef unless (&get_mac($1,$secret) eq $2) ;
 		
 	    my %alt_emails ;
@@ -242,7 +279,7 @@ sub set_cookie_extern {
     my $emails = join(',',@mails);
 
     $value = sprintf '%s&%s',$emails,&get_mac($emails,$secret);
-  
+ 
     if ($http_domain eq 'localhost') {
 	$http_domain="";
     }
@@ -263,7 +300,7 @@ sub set_cookie_extern {
 sub get_which_cookie {
     
     my $http_cookie = shift;
-    &main::wwslog('debug2',"get_which_cookie ($http_cookie)");    
+    &do_log('debug2',"get_which_cookie ($http_cookie)");    
 
     my %cookies = parse CGI::Cookie($http_cookie);
         
@@ -290,7 +327,7 @@ sub set_which_cookie {
     if ($domain eq 'localhost') {
 	$domain="";
     }
-    &main::wwslog('debug2',"set_which_cookie ($domain,$commawhich)");
+    &do_log('debug2',"set_which_cookie ($domain,$commawhich)");
 
     my $cookie = new CGI::Cookie (-name    => 'your_subscriptions',
 				  -value   => $commawhich ,
@@ -303,6 +340,108 @@ sub set_which_cookie {
     return 1;
 }
 
+## get unappropriate_cas_server
+sub get_do_not_use_cas {
+    
+    my $http_cookie = shift;
+
+    my %cookies = parse CGI::Cookie($http_cookie);
+        
+    foreach (keys %cookies) {
+	my $cookie = $cookies{$_};
+	
+	next unless ($cookie->name eq 'do_not_use_cas');
+	my $val = $cookie->value;
+	return ($cookie->value);
+    }
+
+    return (undef);
+}
+
+## Set cookie for accessing web archives
+sub set_do_not_use_cas {
+    my $domain = shift;
+    my $value = shift ;    
+    my $expires=shift;
+    
+    my $expiration;
+
+    if ($expires =~ /now/i) {
+	$expiration = "-10y";
+    }else{
+	$expiration = '+'.$expires.'m';
+    }
+
+    if ($domain eq 'localhost') {
+	$domain="";
+    }
+
+    do_log('debug',"cookielib::set_do_not_use_cas($domain,$value,$expiration) ");
+
+    unless (($value == 0) || ($value == 1)) {
+	do_log('err',"cookielib::set_do_not_use_cas($value) incorrect parameter");
+	return undef;
+    }
+    
+    my $cookie = new CGI::Cookie (-name    => 'do_not_use_cas',
+				  -value   => $value ,
+				  -domain  => $domain,
+				  -expires => $expiration,
+				  -path    => '/'
+				  );
+    
+    ## Send cookie to the client
+    printf "Set-Cookie:  %s\n", $cookie->as_string;
+    return 1;
+}
+
+
+## Set cookie for accessing web archives
+sub set_cas_server {
+    my $domain = shift;
+    my $value = shift ;    
+    my $expires=shift;
+
+    if ($expires =~ /now/i) {
+	$expires = "-10y";
+    }
+
+    if ($domain eq 'localhost') {
+	$domain="";
+    }
+
+    do_log('debug',"cookielib::set_cas_server($domain,$value) ");
+    
+    my $cookie = new CGI::Cookie (-name    => 'cas_server',
+				  -value   => $value ,
+				  -domain  => $domain,
+				  -expires => $expires,
+				  -path    => '/'
+				  );
+    
+    ## Send cookie to the client
+    printf "Set-Cookie:  %s\n", $cookie->as_string;
+    return 1;
+}
+
+
+## get unappropriate_cas_server
+sub get_cas_server {
+    
+    my $http_cookie = shift;
+
+    my %cookies = parse CGI::Cookie($http_cookie);
+        
+    foreach (keys %cookies) {
+	my $cookie = $cookies{$_};
+	
+	next unless ($cookie->name eq 'cas_server');
+	my $val = $cookie->value;
+	return ($cookie->value);
+    }
+
+    return (undef);
+}
 
 1;
 
